-- ############################################################################# 
-- # ECU data display
-- # 
-- # fuel gauge, turbine status and dual voltage display
-- # Designed for JetiOrbitECUSensor
-- # https://sourceforge.net/projects/orbit-ecu-telemetry-sensor-lib
-- #
-- # Copyright (c) 2017, Bernd Woköck
-- #                       
-- # Released under MIT-license 
-- #                       
-- # 1.00 - Initial release
-- # 1.01 - Adjustable tank size
-- #        Volume on motor control
-- # 1.02 - Bug with volume control at startup removed
-- #                       
-- #############################################################################

-- Config
local sensorId=0
local warningPercent=15
local testMode=false
local tankSize = 1700
local volumeOnMotor  = false --  automatic volume control
local motorOffVolume = 11 -- audio volume level when motor off
local motorOnVolume  = 11 -- audio volume level when motor on

-- module locals
local sensorsAvailable = {}
local initAnimation = true
local testModeIndex    -- click test mode
local volOnMotorIndex  -- click volume on motor
local volMotorOnIndex  -- intbox motor on
local volMotorOffIndex -- intbox motor off
local motorIsOn = false

-- sensor values
local fuel = 0
local u_rc = 0
local u_ecu = 0
local rpm = 0
local status = 0

local function DrawFuelGauge(percentage, ox, oy) 
  -- triangle
  lcd.drawLine(6+ox,5+oy,17+ox,5+oy)
  lcd.drawLine(17+ox,5+oy,17+ox,63+oy)
  lcd.drawLine(17+ox,63+oy,14+ox,63+oy)
  lcd.drawLine(14+ox,63+oy,6+ox,5+oy)
  
  -- gas station symbol
  lcd.drawRectangle(51+ox,31+oy,5,9)  
  lcd.drawLine(52+ox,34+oy,54+ox,34+oy)
  lcd.drawLine(50+ox,39+oy,56+ox,39+oy)
  lcd.drawLine(57+ox,31+oy,59+ox,33+oy)
  lcd.drawLine(59+ox,33+oy,59+ox,37+oy)
  lcd.drawPoint(58+ox,38+oy)  
  lcd.drawLine(57+ox,38+oy,57+ox,35+oy)  
  lcd.drawPoint(56+ox,35+oy)  
  lcd.drawText(51+ox,2+oy, "F", FONT_MINI)  
  lcd.drawText(51+ox,54+oy, "F", FONT_MINI)  

  -- fuel bar 
  lcd.drawRectangle (21+ox,53+oy,25,11)  -- lowest bar segment
  lcd.drawRectangle (21+ox,41+oy,25,11)  
  lcd.drawRectangle (21+ox,29+oy,25,11)  
  lcd.drawRectangle (21+ox,17+oy,25,11)  
  lcd.drawRectangle (21+ox,5+oy,25,11)   -- uppermost bar segment
  
  -- calc bar chart values
  local nSolidBar = math.floor( percentage / 20 )
  local nFracBar = (percentage - nSolidBar * 20) / 20  -- 0.0 ... 1.0 for fractional bar
  local i
  -- solid bars
  for i=0, nSolidBar - 1, 1 do 
    lcd.drawFilledRectangle (21+ox,53-i*12+oy,25,11) 
  end  
  --  fractional bar
  local y = math.floor( 53-nSolidBar*12+(1-nFracBar)*11 + 0.5)
  lcd.drawFilledRectangle (21+ox,y+oy,25,11*nFracBar) 

  -- numerical percentage display
  -- if( percentage <= 80 ) then 
  --   lcd.drawText(23+ox,4+oy, string.format("%d%s",percentage,"%"), FONT_MINI)  
  -- end	
end

local function DrawTurbineStatus(status, rpm, ox, oy) 
 t = { "-off", "spin", "idle", "run-", "rel-","acce", "heat",
       "stop", "cool", "lock", "runR", "fire",
	   "badG", "badS", "time", "badP", "minT", "minR", "runT",
	   "maxT", "ovrR", "outR", "loNC", "?R/C", "hot-", "loUb" }
  lcd.drawText(4+ox,2+oy, "Turbine", FONT_MINI)  

  if (status ~= 2 ) then
	status = status + 1
	local text = t[status]
	if( text == nil or text == '' ) then text = "???" end
	lcd.drawText(4+ox,15+oy, text, FONT_BOLD)  
  else
	lcd.drawText(4+ox,15+oy, string.format("%.1f",rpm), FONT_BOLD)  
	lcd.drawText(35+ox,19+oy, "T/min", FONT_MINI)  
  end	  
end

local function DrawBattery(u_rc, u_ecu, ox, oy) 
  lcd.drawText(4+ox,1+oy, "RC", FONT_MINI)  
  lcd.drawText(40+ox,1+oy, "ECU", FONT_MINI)  
  lcd.drawText(4+ox,12+oy, string.format("%.1f%s",u_rc,"V"), FONT_BOLD)  
  lcd.drawText(40+ox,12+oy, string.format("%.1f%s",u_ecu,"V"), FONT_BOLD)  
end

local function DrawFuelLow(fuelPercentage, ox, oy) 

  if( system.getTime() % 2 == 0) then -- blink every second
    -- triangle
	lcd.drawLine(6+ox,47+oy,32+ox,3+oy)
	lcd.drawLine(32+ox,3+oy,60+ox,47+oy)
	lcd.drawLine(60+ox,47+oy,6+ox,47+oy)

	lcd.drawLine(7+ox,47+oy,33+ox,3+oy)
	lcd.drawLine(33+ox,3+oy,61+ox,47+oy)
	lcd.drawLine(61+ox,46+oy,6+ox,46+oy)

	lcd.drawLine(8+ox,47+oy,34+ox,3+oy)
	lcd.drawLine(34+ox,3+oy,62+ox,47+oy)
	lcd.drawLine(62+ox,46+oy,8+ox,46+oy)
	lcd.drawText(31+ox,10+oy, "!", FONT_BIG)  
  end  
  
  -- percentage and warning
  local percentX = 20
  if( fuelPercentage < 10 ) then percentX = 23 end
  lcd.drawText(percentX+ox,28+oy, string.format("%d%s",fuelPercentage,"%"), FONT_BOLD)  
  lcd.drawText(1+ox,49+oy, "Fuel Low", FONT_BOLD)  
  
end

-- set volume depending on motor on/off
local function SetVolumeOnMotor() 

  local motor = (status == 2 and rpm > 10)
  if( not motorIsOn and motor ) then
	 system.setProperty("Volume", motorOnVolume)
	 system.messageBox(string.format("Motor on - volume %d",motorOnVolume),2)
	 motorIsOn = true
  elseif( motorIsOn and not motor ) then	 
	 system.setProperty("Volume", motorOffVolume)
	 system.messageBox(string.format("Motor off - volume %d",motorOffVolume),2)
	 motorIsOn = false
  end

end

-- Print the telemetry values
local function OnPrint(width, height) 
  
  -- animation demo at startup - fill tank up
  if( initAnimation ) then
    fuel = fuel - 50
	rpm  = rpm - 2
    if( fuel <= 0 ) then
  	  initAnimation = false
	  status = 0
    end
  end	
  
  -- fuel test on P8
  if( testMode and initAnimation == false ) then
    local val = (-system.getInputs( "P8" ) + 1.0)/2 -- 0...1
    fuel   = val * tankSize
	u_rc   = 8.4 * (1-val)
	u_ecu  = 8.4 * (1-val)
	status = math.floor( 25*(1-val) )
	rpm    = 90
  end 
  
  -- fuel - 1700 (=tankSize) ml is 0%
  local fuelPercentage = (tankSize - fuel )/(tankSize/100)
  if( fuelPercentage > 99 ) then fuelPercentage = 99 end
  if( fuelPercentage < 0 ) then fuelPercentage = 0 end
  if( fuelPercentage > warningPercent or initAnimation ) then
    DrawFuelGauge(fuelPercentage, 1, 0)   
  else
    DrawFuelLow(fuelPercentage, 1, 0) 
  end  

  -- field separator lines
  lcd.drawLine(70,2,70,66)  
  lcd.drawLine(70,36,148,36)  

  -- turbine
  DrawTurbineStatus(status, rpm, 74, 0) 

  -- battery
  DrawBattery(u_rc, u_ecu, 74, 37) 
  
  -- motor dependent volume control
  if( not initAnimation and volumeOnMotor ) then
    SetVolumeOnMotor()
  end
  
end

-------------------------------------------------------------------- 
-- Form functions
--------------------------------------------------------------------
local function sensorChanged(value)

  if(value and value >=0) then 
    sensorId=sensorsAvailable[value].id
  else 
    sensorId = 0
  end     
  system.pSave("SensorId",sensorId)      

end

local function warningChanged(value)
  warningPercent = value
  system.pSave("WarnigPercent",warningPercent)      
end

local function tankSizeChanged(value)
  tankSize = value
  system.pSave("TankSize",tankSize)      
end

local function motorOnVolumeChanged(value)
  motorOnVolume = value
  system.pSave("MotorOnVolume",motorOnVolume)      
end

local function motorOffVolumeChanged(value)
  motorOffVolume = value
  system.pSave("MotorOffVolume",motorOffVolume)      
end

local function volumeOnMotorClicked(value)
  volumeOnMotor = not value
  form.setValue(volOnMotorIndex,volumeOnMotor)
  if( volumeOnMotor ) then
    system.pSave("VolumeOnMotor",1)      
    form.setProperties (volMotorOnIndex, {enabled=true})  
    form.setProperties (volMotorOffIndex, {enabled=true})  
  else
    system.pSave("VolumeOnMotor",0)      
    form.setProperties (volMotorOnIndex, {enabled=false})  
    form.setProperties (volMotorOffIndex, {enabled=false})  
  end
end

local function testModeClicked(value)
  testMode = not value
  form.setValue(testModeIndex,testMode)
end

-- Init Form
local function initForm(formID)

  sensorsAvailable = {}
  local available = system.getSensors();
  local list={}
  local curIndex=-1
  local descr = ""
  for index,sensor in ipairs(available) do 
    if(sensor.param == 0) then
      list[#list+1] = sensor.label
      sensorsAvailable[#sensorsAvailable+1] = sensor
      if(sensor.id==sensorId ) then
        curIndex=#sensorsAvailable
      end 
    end 
  end
  form.addSpacer(100,10)
  form.addLabel({label="Lama Sensor",font=FONT_BOLD})
  -- sensor select
  form.addRow(2)
  form.addLabel({label="Select sensor",width=120})
  form.addSelectbox (list, curIndex,true,sensorChanged,{width=190})
  -- tank volume
  form.addRow(2)
  form.addLabel({label="Tank size (ml)", width=120})
  form.addIntbox(tankSize,100,3000,1700,0,50,tankSizeChanged,{width=190})  
  -- fuel warning threshold
  form.addRow(2)
  form.addLabel({label="Fuel warning (%)", width=130})
  form.addIntbox(warningPercent,0,100,15,0,5,warningChanged,{width=190})  
  -- volume when motor is on
  form.addRow(2)
  form.addLabel({label="Set volume on motor", width=230})
  volOnMotorIndex = form.addCheckbox(volumeOnMotor,volumeOnMotorClicked)  
  form.addRow(4)
  form.addLabel({label="            Motor off:", width=130})
  volMotorOffIndex = form.addIntbox(motorOffVolume,0,16,11,0,1,motorOffVolumeChanged,{width=50})  
  form.addLabel({label="Motor on:", width=80})
  volMotorOnIndex = form.addIntbox(motorOnVolume,0,16,11,0,1,motorOnVolumeChanged,{width=50})  
  if( not volumeOnMotor ) then
    form.setProperties (volMotorOnIndex, {enabled=false})  
    form.setProperties (volMotorOffIndex, {enabled=false})  
  end
  -- test mode - use P8 to change values
  form.addRow(2)
  form.addLabel({label="Test mode (P8)", width=230})
  testModeIndex = form.addCheckbox(testMode,testModeClicked)  
  
end  

-------------------------------------------------------------------- 
-- Init
--------------------------------------------------------------------
local function init() 

  -- sensor id
  sensorId = system.pLoad("SensorId",0)
  if sensorId == 0 then
    local available = system.getSensors(); 
    for index,sensor in ipairs(available) do  
      if((sensor.id & 0xFFFF) == 41993 ) then -- Fill default sensor ID for MegaSensor
        sensorId = sensor.id
        break
      end 
    end
  end

  -- other settings  
  tankSize       = system.pLoad("TankSize",1700)
  warningPercent = system.pLoad("WarningPercent",15)
  motorOnVolume  = system.pLoad("MotorOnVolume",11)
  motorOffVolume = system.pLoad("MotorOffVolume",11)
  volumeOnMotor  = system.pLoad("VolumeOnMotor",0) == 1

  -- init values for startup animation
  initAnimation = true
  fuel   = tankSize   -- empty 
  status = 2          -- running
  rpm    = 90         -- 90 k/min
  
  system.registerTelemetry( 2, "Fuel/ECU/Voltage", 2, OnPrint)  
  system.registerForm(1,MENU_TELEMETRY,"Fuel/ECU/Voltage",initForm,nil,printForm)
end

-------------------------------------------------------------------- 
-- Loop
--------------------------------------------------------------------
local function loop() 
	
  local sensor
  -- fuel
  sensor = system.getSensorByID(sensorId,4)
  if( sensor and sensor.valid ) then fuel   = sensor.value end
  -- RC voltage
  sensor = system.getSensorByID(sensorId,11)
  if( sensor and sensor.valid ) then u_rc   = sensor.value end
  -- ECU voltage
  sensor = system.getSensorByID(sensorId,1)
  if( sensor and sensor.valid ) then u_ecu  = sensor.value end
  -- RPM
  sensor = system.getSensorByID(sensorId,3)
  if( sensor and sensor.valid ) then rpm    = sensor.value end
  -- ECU status
  sensor = system.getSensorByID(sensorId,5)
  if( sensor and sensor.valid ) then status = sensor.value end
  
end

--------------------------------------------------------------------------------
return {init=init, loop=loop, author="Bernd Woköck", version="1.02",name="ECU Data"}

